/*
 * Copyright (C) 2013 Matthew Dawson <jiangy76@mcmaster.ca>
 * Copyright (C) 2014 Yanjun Jiang <jiangy76@mcmaster.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.cvc3generator;

import ca.mcscert.jtet.expression.EnumerationVariableType;
import ca.mcscert.jtet.expression.TypeDeclarationGenerator;
import ca.mcscert.jtet.expression.VariableType;

import java.util.Set;

/**
 * @author Matthew Dawson
 * @author Yanjun  Jiang
 */
final public class CVC3TypeDeclarationGenerator implements TypeDeclarationGenerator {
    @Override
    public String GenerateTypeDeclaration(Set<VariableType> types) {
        StringBuilder ret = new StringBuilder();
        // Generate all the enumeration types.
        for(VariableType type : types) {
            if (type instanceof EnumerationVariableType) {
                EnumerationVariableType enumType = (EnumerationVariableType) type;
                ret.append("DATATYPE\n  ")
                        .append((enumType).name())
                        .append(" = ");

                boolean firstValue = false;
                for(String enumValue : enumType.enumerationValues()) {
                    if (firstValue == true) {
                        ret.append(" | ");
                    }
                    firstValue = true;
                    ret.append(enumValue);
                }
                ret.append("\nEND;\n");
            }
        }

        return ret.toString();
    }
}
