/*
 * Copyright (C) 2014 Yanjun Jiang <jiangy76@mcmaster.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.eventbgenerator;

import ca.mcscert.jtet.tablularexpression.Table;
import java.io.*;

/**
 *
 * @author Yanjun Jiang <jiangy76@mcmaster.ca>
 */
final public class EventBProject {
    public EventBProject(String projectName) {
       m_projectName = projectName;
    }

    public void setTable(Table table) {
       m_table = table;
    }

    public void generate(String folderPath, RefinementMode refineMode) throws IOException {
        //create Project Folder and Project File
        createEventProjectFolder(folderPath);
        writeProjectFile(m_projectName, folderPath);

        //generate Table EventB File
        m_eventBTableGenerator = new EventBTableGenerator(m_table, refineMode);
        m_eventBTableGenerator.generateEventBXml(folderPath);
    }

    private String constructProjectFile(String projectName){
        return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<projectDescription>\n" +
                "        <name>" + projectName + "</name>\n" +
                "        <comment></comment>\n" +
                "        <projects>\n" +
                "        </projects>\n" +
                "        <buildSpec>\n" +
                "                <buildCommand>\n" +
                "                        <name>org.rodinp.core.rodinbuilder</name>\n" +
                "                        <arguments>\n" +
                "                        </arguments>\n" +
                "                </buildCommand>\n" +
                "        </buildSpec>\n" +
                "        <natures>\n" +
                "                <nature>org.rodinp.core.rodinnature</nature>\n" +
                "        </natures>\n" +
                "</projectDescription>\n";
    }

    private void writeProjectFile(String projectName, String folderPath) throws IOException {
        String fileContent = constructProjectFile(projectName);
        String fileName = folderPath + ".project";

        EventBFileWriter.writeEventBBumFile(fileName ,fileContent);
    }

    private static void  createEventProjectFolder(String folderPath) throws IOException {
        if(!(new File(folderPath).isDirectory())) {
            if (!(new File(folderPath).mkdirs())){
                throw new IOException("Failed to create project folder");
            }
        }
    }

    Table m_table;
    String m_projectName;
    EventBTableGenerator m_eventBTableGenerator;
}
