/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import java.util.Arrays;
import java.util.Collection;

/**
 *
 * @author matthew
 */
public enum BinaryOperation {

    Addition,
    Minus,
    Multiplication,
    Division,
    Exponentiation,

    GreaterThen,
    GreaterThenEqual,
    LessThen,
    LessThenEqual,
    Equals,
    NotEquals,

    Implies,
    BooleanAnd,
    BooleanOr;

    Collection<VariableTypeMarker> getInputTypeForOutput(VariableTypeMarker outputType) {
        // Boolean inputs only work with a boolean input!
        if (this == BooleanAnd || this == BooleanOr || this == Implies || this == GreaterThen || this == GreaterThenEqual ||
                this == LessThen || this == LessThenEqual || this == Equals || this == NotEquals) {
            if (!outputType.equals(BooleanVariableType.Type)) {
                throw new IllegalArgumentException("Boolean operators and comparisons require a boolean output type!");
            }
        }
        if (outputType instanceof  EnumerationVariableType) {
            if (this != Equals && this != NotEquals) {
                throw new IllegalArgumentException("Enumeration types can only be checked for equality!");
            }
        }

        // Comparisions produce boolean outputs!
        switch(this) {
            case GreaterThen:
            case GreaterThenEqual:
            case LessThen:
            case LessThenEqual:
                return Arrays.asList(new VariableTypeMarker[]{RealVariableType.Type, FixedPointVariableType.TypeMarker});
            case Equals:
            case NotEquals:
                return Arrays.asList(new VariableTypeMarker[]{RealVariableType.Type, FixedPointVariableType.TypeMarker, EnumerationVariableType.TypeMarker});
        }
        return Arrays.asList(new VariableTypeMarker[]{outputType});
    }

    public VariableType getOutputForInput(VariableType inputType) {
        // Boolean inputs only work with a boolean input!
        if (this == BooleanAnd || this == BooleanOr || this == Implies) {
            if (!inputType.equals(BooleanVariableType.Type)) {
                throw new IllegalArgumentException("Boolean operators require a boolean input type!");
            }
        }

        // Comparisions produce boolean outputs!
        switch(this) {
            case GreaterThen:
            case GreaterThenEqual:
            case LessThen:
            case LessThenEqual:
            case Equals:
            case NotEquals:
                return BooleanVariableType.Type;
        }
        return inputType;
    }
}
