grammar PVSSimpleParser;

/* This is a simple PVS syntax parser.  It is designed to be able to parser simple math statements from PVS, instead of
a full blown parser that could be used to re-implement PVS.  This file is roughly based on MatlabParser.g4, without all
the extra pieces from it.  It is also modified to make the associated pieces in Java simpler. */

//
// ===============================
//

// a precedence hierarchy for parsing expressions

// these are groups of operators that have equivalent precedences
g1: IMPLIES;
g2: LOG_OR;
g3: LOG_AND;
g4: NEG;
g5: ( NEQ | EQ | GRTE | GRT | LSTE | LST );
g6: ( PLUS | MINUS );
g7: ( LEFTDIV | TIMES );
g8: MINUS;
g9: EXPONENTIATION;

// the hierarchy is defined from LOWEST to HIGHEST priority.

expression : e0;

e0	: e1 g1 e0 | e1; // Implies
e1	: e2 g2 e1 | e2; // Logic OR
e2	: e3 g3 e2 | e3; // Logc AND
e3	: g4 e4 | e4; // Logc NOT
e4	: e4 g5 e5 | e5; // Comparison
e5	: e5 g6 e6 | e6; // Addition/Subtraction
e6	: e6 g7 e7 | e7; // Multiplication/Division
e7	: g8 e8 | e8; // Unary Minus
e8	: e8 g9 e9 | e9; // note: in PVS, exponentiation is left-associative
e9	: unary_expression;

unary_expression
	: base_expression #BASE_EXPRESIION
	| LPAREN expression RPAREN # PARENS_EXPRESSION
	;

base_expression
	: id
	| literal_number
	;

literal_number
        : INT
        | FLOAT
        ;

id:	i1=ID;

//
// operators and assignments
//

IMPLIES : '=>';
LOG_OR	: 'OR';
LOG_AND	: 'AND';
LSTE	: '<=';
GRTE	: '>=';
NEQ	: '/=';

EQ	: '=';

NEG	:   'NOT';

LST	: '<';
GRT	: '>';


PLUS	: '+';
MINUS	: '-';
TIMES	: '*';
LEFTDIV	: '/';

EXPONENTIATION: '^';


LPAREN	: '(';
RPAREN	: ')';

ID	: [a-zA-Z][a-zA-Z0-9]*; //Variable identifier.  Start with letter, then alpha numerical allowed.

INT	: '0'..'9'+ ;

FLOAT
	: ('0'..'9')+ '.' ('0'..'9')* EXPONENT?
	| '.' ('0'..'9')+ EXPONENT?
	| ('0'..'9')+ EXPONENT
	;

WS
	: ( ' '
        | '\t'
        | '\n'
        ) -> skip
	;

fragment
EXPONENT
	: ('e'|'E') ('+'|'-')? ('0'..'9')+ ;
