/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package ca.mcscert.jtet.salgenerator;

import ca.mcscert.jtet.expression.*;
import ca.mcscert.jtet.parsers.MatlabParser;
import ca.mcscert.jtet.tablularexpression.Cell;
import ca.mcscert.jtet.tablularexpression.HierarchicalGridDepthFirstCheckerGenerator;
import ca.mcscert.jtet.tablularexpression.TwoDimensionalGrid;

import java.util.List;
import java.util.Map;

/**
 * Created by matthew on 6/3/14.
 */
public class HierarchicalGridSALGenerator implements HierarchicalGridDepthFirstCheckerGenerator {

    public HierarchicalGridSALGenerator(VariableCollection variableDefinitions, Variable outputVariable)
    {
        this.variableDefinitions = variableDefinitions;
        this.m_outputVariable = outputVariable;
    }

    @Override
    public void descendIntoGridFromCell(Cell cell)
    {
        outputCellPrelude(cell);
        m_firstCell = true;
    }

    @Override
    public void ascendFromGrid()
    {
        handleEndIf();
        result.append(" ");
        m_firstCell = false;
    }

    private void outputCellPrelude(Cell cell)
    {
        m_currentlyRunning = true;
        if (m_firstCell) {
            result.append("IF ");
        } else {
            result.append("ELSIF ");
        }
        m_firstCell = false;
        result.append(MatlabParser.parseMatlabCode(variableDefinitions, cell.contents()).getCheckerOutput(m_generator, BooleanVariableType.Type));
        result.append(" THEN ");
    }

    @Override
    public void handleLeafCell(Cell cell, Map<Variable, Cell> outputCells)
    {
        if (m_firstOutputCell == null) {
            m_firstOutputCell = outputCells.get(m_outputVariable);
        }
        outputCellPrelude(cell);
        result.append(getOutputCellAsSALCode(outputCells.get(m_outputVariable)));
        result.append(" ");
    }

    @Override
    public void handleEdgeCell(Cell cell)
    {
    }

    @Override
    public String getFinalString()
    {
        if (m_currentlyRunning && result.length() != 0) {
            handleEndIf();
            m_currentlyRunning = false;
        }
        return result.toString();
    }

    private void handleEndIf() {
        result.append("ELSE ")
                .append(getOutputCellAsSALCode(m_firstOutputCell))
                .append(" ENDIF");
    }

    private String getOutputCellAsSALCode(Cell outputCell) {
        return MatlabParser.parseMatlabCode(variableDefinitions, outputCell.contents()).getCheckerOutput(m_generator, m_outputVariable.type());
    }

    final private StringBuilder result = new StringBuilder();

    private boolean m_currentlyRunning = false;
    private boolean m_firstCell = true;
    private final Variable m_outputVariable;
    private Cell m_firstOutputCell;

    final private VariableCollection variableDefinitions;

    final private SALExpressionGenerator m_generator = new SALExpressionGenerator();
}
