/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.salgenerator;

import ca.mcscert.jtet.expression.*;

import java.util.Map;

/**
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
public class SALExpressionGenerator implements ExpressionGenerator {
    public final static SALExpressionGenerator Generator = new SALExpressionGenerator();

    private static String GetInfixSymbolFor(BinaryOperation op) {
        switch (op) {
            case Addition:
                return "+";
            case Minus:
                return "-";
            case Multiplication:
                return "*";
            case Division:
                return "/";
            case Exponentiation:
                return "^";
            case GreaterThen:
                return ">";
            case GreaterThenEqual:
                return ">=";
            case LessThen:
                return "<";
            case LessThenEqual:
                return "<=";
            case Equals:
                return "=";
            case NotEquals:
                return "/=";
            case BooleanAnd:
                return "AND";
            case BooleanOr:
                return "OR";
            case Implies:
                return "=>";
        }
        throw new RuntimeException("Should never happen!");
    }

    @Override
    public String GenerateBinaryOperation(BinaryOperation op, String lhsExp, String rhsExp, VariableType usedType) {
        return "(" + lhsExp + " " + GetInfixSymbolFor(op) + " " + rhsExp + ")";
    }

    private static String GetUnaryOperationSymbolFor(UnaryOperation op) {
        switch (op) {
            case Negation:
                return "NOT";
            case Minus:
                return "-";
        }
        throw new RuntimeException("Should never happen!  Asked for unhandled op: " + op + "!");
    }

    @Override
    public String GenerateUnaryOperation(UnaryOperation op, String expression, VariableType usedType) {
        return "(" + GetUnaryOperationSymbolFor(op) + " " + expression + ")";
    }

    @Override
    public String GenerateLiteralValue(String value, VariableType requestedType) {
        if (requestedType instanceof RealVariableType) {
            return value;
        } else {
            throw new IllegalArgumentException("Can't cast literal to that type!");
        }
    }
}
