/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.tablularexpression;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
public final class TwoDimensionalGrid {
    // Creates a 1x1 grid.
    public TwoDimensionalGrid() {
        this(1, 1);
    }

    public TwoDimensionalGrid(int sizeX, int sizeY) {
        resize(sizeX, sizeY);

    }

    // Note that this class assumes the grid is never empty.
    // Whenever logically possible, cell contents are preserved at the correct location.
    public void resize(int sizeX, int sizeY) {
        if (sizeX <= 0 || sizeY <= 0) {
            throw new IllegalArgumentException("This grid must have positive sizes in both dimensions!  Given (" +
                    sizeX + ", " + sizeY + ").");
        }
        if (m_grid.size() > sizeX) {
            m_grid.subList(sizeX, m_grid.size()).clear();
        } else {
            while(m_grid.size() != sizeX) {
                m_grid.add(new ArrayList<Cell>());
            }
        }
        for(int i = 0; i < sizeX; ++i) {
            List<Cell> currentList = m_grid.get(i);
            if(currentList.size() > sizeY) {
                currentList.subList(sizeY, currentList.size()).clear();
            } else {
                while(currentList.size() != sizeY) {
                    currentList.add(new Cell());
                }
            }
        }
    }

    public int sizeX() {
        return m_grid.size();
    }

    public int sizeY() {
        return m_grid.get(0).size();
    }

    public Cell get(int x, int y) {
        return m_grid.get(x).get(y);
    }

    // Indexed as m_grid[x][y], for reasons.  All access should be hidden behind api, so that can be changed if
    // performance suffers.
    private List<List<Cell>> m_grid = new ArrayList<List<Cell>>();
}
