/*
 * Copyright (C) 2014 Yanjun Jiang <jiangy76@mcmaster.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.expression;

import ca.mcscert.jtet.cvc3generator.CVC3TypeDeclarationGenerator;
import ca.mcscert.jtet.smtlibchecker.SMTLIBTypeDeclarationGenerator;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

import java.util.*;

import static org.junit.Assert.assertEquals;

/**
 * Provides a set of tests for implementations of {@link TypeDeclarationGenerator}.
 * @author      Yanjun Jiang
 */
@RunWith(Parameterized.class)
public class GenericTypeDeclarationGeneratorTest {
    @Parameters
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
                {new CVC3TypeDeclarationGenerator(),
                        "DATATYPE\n  Enum1 = e1 | e2 | e3\nEND;\n",
                },
                {new SMTLIBTypeDeclarationGenerator(),
                        "(declare-sort Enum1 0)\n" +
                                "(declare-fun e1 () Enum1)\n" +
                                "(declare-fun e2 () Enum1)\n" +
                                "(declare-fun e3 () Enum1)\n" +
                                "(assert (distinct e1 e2 e3))\n" +
                                "(define-fun isEnum1 ((val Enum1)) Bool (or (= val e1) (= val e2) (= val e3)))\n",
                },
        });
    }
    @Parameter(value = 0)
    public TypeDeclarationGenerator generator;

    @Parameter(value = 1)
    public String SingleEnumerationExpectedOutput;

    @Test
    /**
     * This function test the single enumeration scenario
     */
    public void SingleEnumerationTest() {
        if (SingleEnumerationExpectedOutput != null) {
            Set<VariableType> types = new LinkedHashSet<VariableType>();
            EnumerationVariableType enum1 = new EnumerationVariableType("Enum1");
            enum1.enumerationValues().add("e1");
            enum1.enumerationValues().add("e2");
            enum1.enumerationValues().add("e3");

            types.add(enum1);
            assertEquals(SingleEnumerationExpectedOutput, generator.GenerateTypeDeclaration(types));
        }
    }
}
