/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package ca.mcscert.jtet.expression.test;

import ca.mcscert.jtet.cvc3generator.CVC3ExpressionGenerator;
import ca.mcscert.jtet.expression.*;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.List;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertThat;

/**
 * Verifies ExpressionGeneratorSpy properly records operations.
 * @author Matthew Dawson
 */
@RunWith(JUnit4.class)
public class ExpressionGeneratorSpyTest {
    /**
     * Verify a given ExpressionUnaryOperation will be correctly recorded.
     */
    @Test
    public void testSimpleUnaryExpression() {
        Expression expression = new ExpressionUnaryOperation(UnaryOperation.Minus, new Literal("5"));
        expression.getCheckerOutput(m_spy, RealVariableType.Type);

        List<ExpressionGeneratorSpy.SpiedBinaryOp> binaryOps = m_spy.getBinaryOps();
        List<ExpressionGeneratorSpy.SpiedUnaryOp> unaryOps = m_spy.getUnaryOps();
        assertThat(unaryOps.size(), is(1));
        assertThat(binaryOps.size(), is(0));
        assertThat(unaryOps.get(0).expression, is("5"));
        assertThat(unaryOps.get(0).op, is(UnaryOperation.Minus));
        assertThat(unaryOps.get(0).usedType, is((VariableType) RealVariableType.Type));
    }

    /**
     * Verify a given ExpressionBinaryOperation will be correctly recorded.
     */
    @Test
    public void testSimpleBinaryExpression() {
        Expression expression = new ExpressionBinaryOperation(new Literal("5"), BinaryOperation.Addition, new Literal("6"));
        expression.getCheckerOutput(m_spy, RealVariableType.Type);

        List<ExpressionGeneratorSpy.SpiedBinaryOp> binaryOps = m_spy.getBinaryOps();
        List<ExpressionGeneratorSpy.SpiedUnaryOp> unaryOps = m_spy.getUnaryOps();
        assertThat(unaryOps.size(), is(0));
        assertThat(binaryOps.size(), is(1));
        assertThat(binaryOps.get(0).lhsExp, is("5"));
        assertThat(binaryOps.get(0).rhsExp, is("6"));
        assertThat(binaryOps.get(0).op, is(BinaryOperation.Addition));
        assertThat(binaryOps.get(0).usedType, is((VariableType) RealVariableType.Type));
    }

    @Before
    public void setUp() {
        m_generator = new CVC3ExpressionGenerator();
        m_spy = new ExpressionGeneratorSpy(m_generator);
    }
    private CVC3ExpressionGenerator m_generator;
    private ExpressionGeneratorSpy m_spy;
}
