/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.parsers.test;

import ca.mcscert.jtet.cvc3generator.CVC3ExpressionGenerator;
import ca.mcscert.jtet.smtlibchecker.SMTLIBExpressionGenerator;
import ca.mcscert.jtet.expression.*;
import ca.mcscert.jtet.parsers.PVSSimpleParser;
import ca.mcscert.jtet.parsers.VariableParser;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;

/**
 *
 * @author matthew
 */
@RunWith(JUnit4.class)
public class PVSSimpleParserTest {
    @Test
    public void testSimpleNumberParse() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "5\n");
        VariableType variableType = RealVariableType.Type;

        assertSame(Literal.class, expr.getClass());
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("5", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("5.0", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testSimpleRealVariableParse() {
        Expression expr = PVSSimpleParser.parsePVSCode(variableCollection, "a\n");

        assertSame(Variable.class, expr.getClass());
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        VariableType variableType = RealVariableType.Type;
        assertEquals("a", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("a", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testSimpleAdditionWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4+5\n");

        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        VariableType variableType = RealVariableType.Type;
        assertEquals("(4 + 5)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(+ 4.0 5.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testSimpleMultiplicationWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4*5\n");

        VariableType variableType = RealVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("(4 * 5)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(* 4.0 5.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testSimpleNegationWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "NOT 0\n");

        VariableType variableType = RealVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("(NOT 0)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(not 0.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testDoubleAdditionWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4+5+6\n");

        VariableType variableType = RealVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("((4 + 5) + 6)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(+ (+ 4.0 5.0) 6.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testMultiplyThenAddWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4*5+6\n");

        VariableType variableType = RealVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("((4 * 5) + 6)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(+ (* 4.0 5.0) 6.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testAddThenMultiplyWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4+5*6\n");

        VariableType variableType = RealVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("(4 + (5 * 6))", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(+ 4.0 (* 5.0 6.0))", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testInsanseExpressionsWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4*5+6*7\n");
        Expression expr2 = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "4+5+6*7*8\n");

        ExpressionGenerator cvc3generator = new CVC3ExpressionGenerator();
        ExpressionGenerator smtlibgenerator = new SMTLIBExpressionGenerator();

        VariableType variableType = RealVariableType.Type;
        assertEquals("((4 * 5) + (6 * 7))", expr.getCheckerOutput(cvc3generator, variableType));
        assertEquals("(+ (* 4.0 5.0) (* 6.0 7.0))", expr.getCheckerOutput(smtlibgenerator, variableType));

        assertEquals("((4 + 5) + ((6 * 7) * 8))", expr2.getCheckerOutput(cvc3generator, variableType));
        assertEquals("(+ (+ 4.0 5.0) (* (* 6.0 7.0) 8.0))", expr2.getCheckerOutput(smtlibgenerator, variableType));
    }

    @Test
    public void testSimpleConditionalsWithLiterals() {
        Expression expr = PVSSimpleParser.parsePVSCode(new VariableCollection(null), "5 > 5\n");

        VariableType variableType = BooleanVariableType.Type;
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("(5 > 5)", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(> 5.0 5.0)", expr.getCheckerOutput(generator, variableType));
    }

    @Test
    public void testInsanseExpressionsWithLiteralsAndVariables() {
        Expression expr = PVSSimpleParser.parsePVSCode(variableCollection, "z = 1 AND x = 0 AND y = 2");

        VariableType variableType = BooleanVariableType.Type;
        assertEquals(variableType, expr.actualOutputTypeForRequestedOutput(variableType));
        ExpressionGenerator generator = new CVC3ExpressionGenerator();
        assertEquals("((z = 1) AND ((x = 0) AND (y = 2)))", expr.getCheckerOutput(generator, variableType));
        generator = new SMTLIBExpressionGenerator();
        assertEquals("(and (= z 1.0) (and (= x 0.0) (= y 2.0)))", expr.getCheckerOutput(generator, variableType));
    }

    VariableParser variableParser = new VariableParser();
    final VariableCollection variableCollection = new VariableCollection(variableParser.parseVariables("a,x,y,z"));
}
