/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.parsers.test;

import ca.mcscert.jtet.expression.*;
import ca.mcscert.jtet.parsers.VariableParser;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.Map;


/**
 *
 * @author matthew
 */
@RunWith(JUnit4.class)
public class VariableParserTest {
    @Test
    public void testSimpleVariable() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a");
        Map<String, Variable> vars = variableCollection.m_variables;
        Variable a = vars.get("a");
        assertEquals("a", a.name());
        assertEquals(RealVariableType.Type, a.type());
        assertThat(variableCollection.m_enumerationTypes.size(), is(0));
    }
    @Test
    public void testRealVariable() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a: real");
        Map<String, Variable> vars = variableCollection.m_variables;
        Variable a = vars.get("a");
        assertEquals("a", a.name());
        assertEquals(RealVariableType.Type, a.type());
    }
    @Test
    public void testBooleanVariable() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a: bool");
        Map<String, Variable> vars = variableCollection.m_variables;
        Variable a = vars.get("a");
        assertEquals("a", a.name());
        assertEquals(BooleanVariableType.Type, a.type());
    }
    @Test
    public void testFixedPointNoDecimalVariable() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a: fixedt(1, 16)");
        Map<String, Variable> vars = variableCollection.m_variables;
        Variable a = vars.get("a");
        FixedPointVariableType type = (FixedPointVariableType)a.type();
        assertEquals("a", a.name());
        assertNotNull(type);
        assertEquals(true, type.isSigned());
        assertEquals(16, type.digits());
        assertEquals(0, type.fractionPart());

        vars = variableParser.parseVariables("a: fixedt(0, 15, 4)").m_variables;
        a = vars.get("a");
        type = (FixedPointVariableType)a.type();
        assertEquals("a", a.name());
        assertNotNull(type);
        assertEquals(false, type.isSigned());
        assertEquals(15, type.digits());
        assertEquals(4, type.fractionPart());
    }
    @Test
    public void testCaseInsensitiveTypeSearch() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a: ReAl");
        Map<String, Variable> vars = variableCollection.m_variables;
        Variable a = vars.get("a");
        assertEquals("a", a.name());
        assertEquals(RealVariableType.Type, a.type());

        vars = variableParser.parseVariables("A: REAL").m_variables;
        a = vars.get("A");
        assertEquals("A", a.name());
        assertEquals(RealVariableType.Type, a.type());
    }
    @Test
    public void testMultiRealVariables() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a,b");
        assertThat(variableCollection.m_variables.size(), is(2));

        Variable var = variableCollection.m_variables.get("a");
        assertThat(var.name(), is("a"));
        assertThat(var.type(), is((VariableType) RealVariableType.Type));

        var = variableCollection.m_variables.get("b");
        assertThat(var.name(), is("b"));
        assertThat(var.type(), is((VariableType) RealVariableType.Type));
    }

    @Test
    public void testMultiMixedVariables() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("a:fixedt(1,16,9),b");
        assertThat(variableCollection.m_variables.size(), is(2));

        Variable var = variableCollection.m_variables.get("a");
        assertThat(var.name(), is("a"));
        assertThat(var.type(), is((VariableType) new FixedPointVariableType(true, 16, 9)));

        var = variableCollection.m_variables.get("b");
        assertThat(var.name(), is("b"));
        assertThat(var.type(), is((VariableType) RealVariableType.Type));
    }
    @Test
    public void testSubtypeTypeExtraction() {
        VariableParser variableParser = new VariableParser();
        PartialVariableCollection variableCollection = variableParser.parseVariables("d:{y|c OR y > 5},a,b:{x:real|x>5},c:bool,z:{x:bool|x /= c}");
        Map<String, Variable> vars = variableCollection.m_variables;
        assertThat(vars.size(), is(5));

        assertThat(vars.get("d").name(), is("d"));
        assertThat(vars.get("d").type(), is((VariableType) RealVariableType.Type));
        assertThat(vars.get("d").type().subtypePredicate().toString(), is("(c BooleanOr (d GreaterThen 5))"));

        assertThat(vars.get("a").name(), is("a"));
        assertThat(vars.get("a").type(), is((VariableType) RealVariableType.Type));
        assertThat(vars.get("a").type().subtypePredicate(), is((Object)null));

        assertThat(vars.get("b").name(), is("b"));
        assertThat(vars.get("b").type(), is((VariableType) RealVariableType.Type));
        assertThat(vars.get("b").type().subtypePredicate().toString(), is("(b GreaterThen 5)"));

        assertThat(vars.get("c").name(), is("c"));
        assertThat(vars.get("c").type(), is((VariableType) BooleanVariableType.Type));
        assertThat(vars.get("c").type().subtypePredicate(), is((Object)null));

        assertThat(vars.get("z").name(), is("z"));
        assertThat(vars.get("z").type(), is((VariableType) BooleanVariableType.Type));
        assertThat(vars.get("z").type().subtypePredicate().toString(), is("(z NotEquals c)"));
    }

    @Test
    public void testCustomType() {
        VariableParser variableParser = new VariableParser();
        variableParser.addCustomType("myreal", new RealVariableType());
        BooleanVariableType customBool = new BooleanVariableType();
        customBool.setSubtypePredicate(new Variable("c", new BooleanVariableType()));
        variableParser.addCustomType("mybool", customBool);

        Map<String, Variable> vars = variableParser.parseVariables("a:myreal,b:mybool").m_variables;
        assertThat(vars.size(), is(2));

        assertThat(vars.get("a").name(), is("a"));
        assertThat(vars.get("a").type(), is((VariableType) RealVariableType.Type));

        assertThat(vars.get("b").name(), is("b"));
        assertThat(vars.get("b").type(), is((VariableType) BooleanVariableType.Type));
        assertThat(vars.get("b").type().subtypePredicate().toString(), is("c"));
    }

    @Test
    public void testUnknownCustomType() {
        VariableParser variableParser = new VariableParser();

        Map<String, Variable> vars = variableParser.parseVariables("a:unknowntype").m_variables;
        assertThat(vars.size(), is(1));

        assertThat(vars.get("a").name(), is("a"));
        assertThat(vars.get("a").type(), is((VariableType) RealVariableType.Type));
    }

    @Test
    public void testEnumerationsAreAvailable() {
        EnumerationVariableType enumType = new EnumerationVariableType("enumtype");
        enumType.enumerationValues().add("enum1");
        enumType.enumerationValues().add("enum2");

        VariableParser variableParser = new VariableParser();
        variableParser.addCustomType("enumtype", enumType);

        PartialVariableCollection vars = variableParser.parseVariables("a:enumtype");

        assertThat(vars.m_variables.size(), is(1));
        assertThat(vars.m_enumerationTypes.size(), is(1));

        assertThat(vars.m_variables.get("a").type(), is((VariableType)enumType));

        assertThat(vars.m_enumerationTypes.toArray(new EnumerationVariableType[]{})[0], is(sameInstance(enumType)));
    }
}
