/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.smtlibchecker;

import ca.mcscert.jtet.expression.VariableCollection;
import ca.mcscert.jtet.parsers.VariableParser;
import ca.mcscert.jtet.tablularexpression.HierarchicalCell;
import ca.mcscert.jtet.tablularexpression.Table;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.List;

import static org.junit.Assert.assertEquals;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
@RunWith(JUnit4.class)
public class HierarchicalGridSMTLIBGeneratorTest {

    // Exercise the smtlib generator using a table designed in matlab.
    @Test
    public void exerciseSMTLIBGenerator() {
        Table table = new Table(null, new VariableCollection(variableParser.parseVariables("x,z")));
        final List<HierarchicalCell> topCells = table.getLeftGrid().getSubHierarchy();

        topCells.add(new HierarchicalCell("x > 0"));
        topCells.add(new HierarchicalCell("x == 0"));
        topCells.add(new HierarchicalCell("0 > x"));

        List<HierarchicalCell> nextGrid = topCells.get(1).getSubHierarchy();
        nextGrid.add(new HierarchicalCell("z > 0"));
        nextGrid.add(new HierarchicalCell("z == 0"));
        nextGrid.add(new HierarchicalCell("0 > z"));

        nextGrid.get(1).getSubHierarchy().add(new HierarchicalCell("z == 0"));

        String out = table.walk(new HierarchicalGridSMTLIBGenerator(table.getVariables()));

        String expected = "(push 1)\n" +
                "(assert (and true (or (and (> x 0.0) (= x 0.0))\n" +
                "(and (> x 0.0) (> 0.0 x))\n" +
                "(and (= x 0.0) (> 0.0 x)))))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and true (not (or (> x 0.0) (= x 0.0) (> 0.0 x)))))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and (= x 0.0) (or (and (> z 0.0) (= z 0.0))\n" +
                "(and (> z 0.0) (> 0.0 z))\n" +
                "(and (= z 0.0) (> 0.0 z)))))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and (= x 0.0) (not (or (> z 0.0) (= z 0.0) (> 0.0 z)))))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and (= z 0.0) (= x 0.0) false))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and (= z 0.0) (= x 0.0) (not (= z 0.0))))\n" +
                "(check-sat)\n" +
                "(pop 1)\n";

        assertEquals(expected, out);
    }

    // Test the smtlib generator using an empty table
    @Test
    public void testEmptyTable() {
        Table table = new Table(null, new VariableCollection(variableParser.parseVariables("x,z")));

        String out = table.walk(new HierarchicalGridSMTLIBGenerator(new VariableCollection(variableParser.parseVariables("x,z"))));

        String expected = "(push 1)\n" +
                "(assert (and true false))\n" +
                "(check-sat)\n" +
                "(pop 1)\n" +
                "(push 1)\n" +
                "(assert (and true (not true)))\n" +
                "(check-sat)\n" +
                "(pop 1)\n";

        assertEquals(expected, out);
    }

    VariableParser variableParser = new VariableParser();
}
