/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.tablularexpression.test;

import ca.mcscert.jtet.tablularexpression.TwoDimensionalGrid;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

import java.util.Arrays;
import java.util.Collection;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
@RunWith(Parameterized.class)
public class TwoDimensionalGridResizeTest {
    @Parameters
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
                {
                        1, 1,
                        4, 5
                },
                {
                        3, 3,
                        5, 3
                },
                {
                        3, 3,
                        3, 5
                },
                {
                        3, 3,
                        5, 3
                },
                {
                        4, 4,
                        2, 2
                },
                {
                        4, 4,
                        2, 4
                },
                {
                        4, 4,
                        4, 2
                },
                {
                        4, 4,
                        2, 6
                },
                {
                        4, 4,
                        6, 2
                }
        });
    }

    @Parameter(value = 0)
    public int initialX;

    @Parameter(value = 1)
    public int initialY;

    @Parameter(value = 2)
    public int newX;

    @Parameter(value = 3)
    public int newY;

    @Test
    public void doTest()
    {
        TwoDimensionalGrid grid = new TwoDimensionalGrid(initialX, initialY);
        //Verify proper creation (should never fail ...)
        assertThat(grid.sizeX(), is(initialX));
        assertThat(grid.sizeY(), is(initialY));
        // Setup cell contents for the next test.
        int i = 0;
        for (int x = 0; x < initialX; ++x) {
            for (int y = 0; y < initialY; ++y) {
                grid.get(x, y).setContents("" + (i++));
            }
        }

        //And do the resize
        grid.resize(newX, newY);
        // Verify new sizes
        assertThat(grid.sizeX(), is(newX));
        assertThat(grid.sizeY(), is(newY));
        // And verify a cell exists at the maximum size.
        assertThat(grid.get(newX - 1, newY - 1), is(notNullValue()));
        // And that the original cells are still there.
        i = -1;
        for (int x = 0; x < initialX; ++x) {
            for (int y = 0; y < initialY; ++y) {
                i++;
                if (x >= newX || y >= newY) {
                    continue;
                }
                assertThat(grid.get(x, y).contents(), is("" + i));
            }
        }
    }
}
